# Gitflow

## Initialize

- Create a new project in gitlab and take note of its url

### Protecting Master Branch

On the gitlab page of your project go to _Settings/Repository_ in the _Protected Branches_ section.
Select the **Master** branch and set **allow to merge** to **_Master_** and **allow to push** to **No one**
This will prevent anyone to push on the master branch, the only way to add code to the branch will be using merge requests.

### Setting Development Branch

- Retrieve the project on your computer, open a command line and execute.

```sh
git clone https://forge.grandlyon.com/web-et-numerique/factory/llle_project/ecolyo
```

- Create the development branch

```sh
git checkout -b dev
```

- Init your project and then type any time you want to commit changes:

```sh
git add .
git commit -m "Commit changes"
git push origin dev
```

## Start Developing

For any other development you should create a branch from the development branch.

First make sure to be on the **_dev_** branch, type:

```sh
git branch
```

> to see your current branch

### Conventional Commits

The commit contains the following structural **types**:

- fix
- feat

In our team organisation we are using Icescrum to keep track of our User Stories (US). Each US is identified by a number. In order to easily identify the commit that refers to the US, a `feature` branch name should follow this structure:

- _feat/USXX-<title_of_the_user_story\>_

You can create the branch by typing:

```sh
git checkout -b feat/USXX-<title_of_the_user_story>
```

> Note that the **feat** type is the only one with a dedicated folder (feat/-US).

Each fix branch should be named as the following _fix/<title_of_the_fix\>_.

You can create the branch by typing:

```sh
git checkout -b fix/-<title_of_the_fix>
```

**Breaking Change**
Appends a `!` before the title of the commit to notify that a breaking change has happened. Give a solid description of the changes as well.

**Types** other than fix and feature are allowed, for example _build_, _chore_, _ci_, _docs_, _style_, _refactor_, _perf_, _test_ and others.

**add tag**

```sh
git tag -a 1.0.0 -m "complementary message"
```

### Gitflow Scheme

![Gitflow](/img/gitflow.png)
